#ifndef SRC_DMX_PHYS_M644_HPP_
#define SRC_DMX_PHYS_M644_HPP_

#include <stdint.h>

#ifndef DMX_BAUD
/// Baud rate of DMX transmission, should always be 250Kb
#define DMX_BAUD 250000UL
#endif

#ifndef DMX_UNIVERSE
/// Size of a DMX universe, should always be 512
#define DMX_UNIVERSE 512
#endif

enum Phys_State {
  DMX_BREAK,
  DMX_MAB,
  DMX_START,
  DMX_DATA
};

/// Physical Layer for DMX Transceiver on ATMega644, uses USART0
class Phys_m644 {
 private:
  class interrupt_container;

  Phys_State tx_state;
  uint8_t tx_startcode;
  uint8_t tx_data[DMX_UNIVERSE];
  uint8_t *tx_pointer;

  void tx_enable(void);
  void tx_disable(void);
  void tx_update_state(void);

  volatile uint8_t tx_elapsed_us;

  void tx_timer_start(void);
  void tx_timer_stop(void);

  Phys_State rx_state;
  uint8_t rx_startcode;
  uint8_t rx_data[DMX_UNIVERSE];
  uint8_t *rx_pointer;

  void rx_enable(void);
  void rx_disable(void);
  void rx_update_state_uart(void);
  void rx_update_state_pcint(void);

  volatile uint8_t rx_elapsed_us;

  void rx_timer_start(void);
  void rx_timer_stop(void);

 public:
  Phys_m644(uint8_t tx_startcode);
  void set(uint16_t addr, uint8_t value);
  void start(void);
  void stop(void);
  ~Phys_m644(void);
};

#endif /* SRC_DMX_PHYS_M644_HPP_ */
